/*
 * Awake File: Easy file upload & download through HTTP with Java
 * Awake SQL: Remote JDBC access through HTTP.                                    
 * Copyright (C) 2011, Kawan Softwares S.A.S.
 * (http://www.awakeframework.org). All rights reserved.                                
 *                                                                               
 * Awake File/SQL is free software; you can redistribute it and/or                 
 * modify it under the terms of the GNU Lesser General Public                    
 * License as published by the Free Software Foundation; either                  
 * version 2.1 of the License, or (at your option) any later version.            
 *                                                                               
 * Awake File/SQL is distributed in the hope that it will be useful,               
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             
 * Lesser General Public License for more details.                               
 *                                                                               
 * You should have received a copy of the GNU Lesser General Public              
 * License along with this library; if not, write to the Free Software           
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  
 * 02110-1301  USA
 *
 * Any modifications to this file must keep this entire header
 * intact.
 */

//HttpParameters.java
//Copyright (c) Kawan Softwares S.A.S, 2011
//
//Last Updates: 
// 14 sept. 2010 18:15:30 Kawan Softwares S.A.S
// 20/09/10 20:05 NDP - New header comments
// 20/11/10 20:40 NDP : HttpProtocolParameters: add parameters of suppressed AwakeSessionConfig class
// 08/06/11 12:20 NDP : HttpProtocolParameters: Clean Javadoc
// 23/11/11 13:05 NDP : HttpProtocolParameters: Rneame encrypt methods getEncryptionpassword()/setEncryptionpassword()
// 05/12/11 15:10 NDP : HttpProtocolParameters: clean Javadoc
// 12/01/12 17:30 NDP : HttpProtocolParameters: comments

package org.awakefw.commons.api.client;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
 * 
 * Allows to define some parameters for the Awake session:
 * <ul>
 * <li>Buffer size when uploading files of Blobs.</li>
 * <li>Buffer size when downloading files of Blobs.</li>
 * <li>Maximum length authorized for a string download, in order to avoid
 * OutOfMemoryException on client side.</li>
 * <li>Password to use for encrypting all parameters request between and Host.</li>
 * </ul>
 * 
 * Allows also to store http protocol parameters that will be passed to the
 * underlying <code>DefaultHttpClient</code> class of the <a
 * href="http://hc.apache.org/httpcomponents-client-ga/">Jakarta HttpClient
 * Library<a> library.
 * <p>
 * Use this class only if you want to change the default values of the
 * HttpClient library and pass the created instance to
 * <code>AwakeFileSession</code> or <code>AwakeConnection</code>.
 * <p>
 * For example, the following change the default connection timeout to 10
 * seconds and the default socket timeout to 60 seconds: <blockquote>
 * 
 * <pre>
 * String url = "https://www.acme.org/AwakeFileManager";
 * String username = "myUsername";
 * char [] password = {'m', 'y', 'P', 'a', 's', 's', 'w', 'o', 'r', 'd'}; 
 * &nbsp;        
 * HttpProtocolParameters httpProtocolParameters = new HttpProtocolParameters();
 * &nbsp;
 * // Sets the timeout until a connection is established to 10 seconds
 * httpProtocolParameters.setHttpClientParameter(
 * "http.connection.timeout", new Integer(10 * 1000));
 * &nbsp;
 * // Sets the socket timeout (SO_TIMEOUT) to 60 seconds
 * httpProtocolParameters.setHttpClientParameter("http.socket.timeout",
 * new Integer(60 * 1000));
 * &nbsp;        
 * // We will use no proxy
 * HttpProxy httpProxy = null;
 * &nbsp;        
 * AwakeFileSession awakeFileSession 
 * = new AwakeFileSession(url, username, password, httpProxy, httpProtocolParameters);
 * &nbsp;
 * // Etc.
 * </blockquote>
 * </pre>
 * 
 * <br>
 * See <a href=
 * "http://hc.apache.org/httpcomponents-client-ga/tutorial/html/fundamentals.html#d4e310"
 * >HttpComponents Tutorial</a> for more info on HTTP parameters.
 * 
 * @author Nicolas de Pomereu
 * @since 1.0
 */

public final class HttpProtocolParameters {

    /** Defines one kilobyte */
    public static final int KB = 1024;
    
    /** Defines one megabyte  */    
    public static final int MB = 1024 * KB;
    
    /** Defines one gigabyte  */      
    public static final long GB = 1024 * MB;
    
    /** The default maximum size of a string read from input stream. */
    public static final int DEFAULT_MAX_LENGTH_FOR_STRING = 200 * KB;

    /** The default buffer size when uploading a file */
    public static final int DEFAULT_UPLOAD_BUFFER_SIZE =  20 * KB;

    /** The default Buffer size for download and copy */
    public static final int DEFAULT_DOWNLOAD_BUFFER_SIZE = 1 * KB;

    /** The maximum size of a string read from input stream. Should be <= 200 Kb */
    private int maxLengthForString = DEFAULT_MAX_LENGTH_FOR_STRING;

    /** The buffer size when uploading a file */
    private int uploadBufferSize = DEFAULT_UPLOAD_BUFFER_SIZE;

    /** Buffer size for download and copy */
    private int downloadBufferSize = DEFAULT_DOWNLOAD_BUFFER_SIZE;

    /**
     * The password to use to encrypt all request parameter names and values.
     * null means no encryption is done
     */
    private char[] encryptionPassword = null;
    
    /** Hash map of HTTP parameters that this collection contains */
    private Map<String, Object> httpClientParameters = null;
    
    /**
     * Constructor
     */
    public HttpProtocolParameters() {
	httpClientParameters = new HashMap<String, Object>();
    }

    /**
     * Returns the Maximum length authorized for a string download, to avoid
     * OutOfMemoryException on client side.
     * 
     * @return the Maximum length authorized for a string download
     */
    public int getMaxLengthForString() {
	return maxLengthForString;
    }

    /**
     * Sets the the Maximum length authorized for a string download, to avoid
     * OutOfMemoryException on client side.
     * 
     * @param maxLengthForString
     *            the Maximum length authorized for a string download
     */
    public void setMaxLengthForString(int maxLengthForString) {
	this.maxLengthForString = maxLengthForString;
    }

    /**
     * Returns the buffer size when uploading files.
     * 
     * @return the buffer size when uploading files
     */
    public int getUploadBufferSize() {
	return this.uploadBufferSize;
    }

    /**
     * Sets the buffer size when uploading files
     * 
     * @param uploadBufferSize
     *            the buffer size when uploading files
     */
    public void setUploadBufferSize(int uploadBufferSize) {
	this.uploadBufferSize = uploadBufferSize;
    }

    /**
     * Returns the buffer size when downloading files.
     * 
     * @return the buffer size when downloading files
     */
    public int getDownloadBufferSize() {
	return this.downloadBufferSize;
    }

    /**
     * Sets the buffer size when downloading files.
     * 
     * @param downloadBufferSize
     *            the buffer size when downloading files to set
     */
    public void setDownloadBufferSize(int downloadBufferSize) {
	this.downloadBufferSize = downloadBufferSize;
    }

    /**
     * Returns the encryption Password that encrypts http request parameters.
     * 
     * @return the encryption Password that encrypts http request parameters
     */
    public char[] getEncryptionPassword() {
	return this.encryptionPassword;
    }

    /**
     * Sets the encryption Password that encrypts http request parameters.
     * 
     * @param encryptionPassword
     *            the encryption Password that encrypts http request parameters
     */
    public void setEncryptionPassword(
	    char[] encryptionPassword) {
	this.encryptionPassword = encryptionPassword;
    }

    
    /**
     * Returns the HttpClient Parameters.
     * 
     * @return the HttpClient Parameters
     */
    public Map<String, Object> getHttpClientParameters() {
	return this.httpClientParameters;
    }

    /**
     * Sets the HttpClient Parameters.
     * 
     * @param httpClientParameters
     *            the HttpClient Parameters. to set
     */
    public void setHttpClientParameters(Map<String, Object> httpClientParameters) {
	this.httpClientParameters = httpClientParameters;
    }

    /**
     * Sets the value of this HttpClient Library parameter
     * 
     * @param name
     *            the parameter name
     * @param value
     *            the parameter value
     * 
     * @throws IllegalArgumentException
     *             if name is null
     */
    public synchronized void setHttpClientParameter(final String name,
	    final Object value) {

	if (name == null) {
	    throw new IllegalArgumentException(
		    "parameter name can not be null!");
	}

	this.httpClientParameters.put(name, value);
    }

    /**
     * Gets the value of this HttpClient Library parameter.
     * 
     * @param name
     *            the parameter name *
     * @return the parameter value
     * 
     * @throws IllegalArgumentException
     *             if name is null
     */
    public synchronized Object getHttpClientParameter(final String name) {

	if (name == null) {
	    throw new IllegalArgumentException(
		    "parameter name can not be null!");
	}

	return this.httpClientParameters.get(name);
    }

    /**
     * Returns the new parameters key set.
     * 
     * @return the new parameters key set.
     * @see java.util.Map#keySet()
     */
    public synchronized Set<String> getHttpClientParameterNames() {
	return this.httpClientParameters.keySet();
    }

    /**
     * Removes all parameters from this collection.
     */
    public void clearHttpClientParameters() {
	this.httpClientParameters.clear();
    }

    /**
     * Returns a clean representation of the <code>HttpProtocolParameters</code> instance.
     * @return a clean representation of the <code>HttpProtocolParameters</code> instance
     */
    @Override
    public String toString() {
	return "HttpProtocolParameters [maxLengthForString="
		+ this.maxLengthForString + ", uploadBufferSize="
		+ this.uploadBufferSize + ", downloadBufferSize="
		+ this.downloadBufferSize + ", encryptionPassword="
		+ Arrays.toString(this.encryptionPassword)
		+ ", httpClientParameters=" + this.httpClientParameters + "]";
    }

}
